#!/usr/bin/perl -wT
# 
# This Quiz application and associated files is
# Copyright (C) 2000, Jan-Pieter Cornet
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use MIME::Base64;
use Storable;
use FileHandle;

# logfile to store progress in
open(LOG, ">>quiz.log");
LOG->autoflush(1);

# initialise CGI processing
my $q = new CGI;

my $ronde = $q->param(-name => "ronde");
unless ( $ronde && $ronde =~ /^(\d+)$/ && -f "ronde$1.data" ) {
    die "Go away, hacker\n";
}
my $data = retrieve "ronde$1.data" or
    die "Cannot retrieve ronde$ronde.data: $!";
# $data is reference to LoL, format of the second list is:
# [0] - vraag
# [1] - image or undef
# [2] - undef if can-reorder, correct answer otherwise (1..4). For can-reorder
#     correct answ is always answ1
# [3] - answ1
# [4] - answ2
# [5] - answ3
# [6] - answ4
# [7] - extra explanation, if any

# get ID
my @cookie;
my $id = getID($q, \@cookie);

# print header, possibly using cookie ID
print $q->header(-cookie => [@cookie]),
    $q->start_html(-title => $data->[0],
		   -BGCOLOR => "black",
		   -LINK => "yellow",
		   -ALINK => "red",
		   -VLINK => "#CCCC00",
		   -TEXT => "white"),
    "\n";

if ( $q->param(-name => "answ") ) {
    process_answers($q, $data, $id, $ronde);
}
else {
    present_questions($q, $data, $id, $ronde);
}

print <<HTML,
<table width="100%" border=0 cellspacing=0 cellpadding=0>
<tr><td bgcolor="#999999" align=center><img src="laga.gif"></td></tr>
<tr><td bgcolor=white><img height=3 src="trans.gif"></td></tr>
<tr><td bgcolor="#ffcc33"><img height=18 src="trans.gif"></td></tr>
<tr><td bgcolor=white><img height=10 src="trans.gif"></td></tr>
<tr><td align=right bgcolor="white"><img src="xslogo40mmkleur3.gif"><img width=10 src="trans.gif"></td></tr>
<tr><td bgcolor=white><img height=84 src="trans.gif"></td></tr>
</table>
HTML
    $q->end_html;

exit;

# gets ID. if necessary, adds cookie to @cookie array
sub getID {
    my($q, $cref) = @_;

    my $id = $q->cookie(-name => "ID") || $q->param(-name => "ID");
    if ( ! $id ) {
	my $ip = $ENV{REMOTE_ADDR} || "0.0.0.0";
	$ip = pack("C*", split /\./, $ip);
	my $port = pack("S", $ENV{REMOTE_PORT} || 0);
	my $time = pack("L", time);
	my $stamp = localtime;
	$id = encode_base64($ip . $port . $time);
	chomp $id;
	push @$cref, $q->cookie(-name => "ID",
			        -value => $id,
			        -expires => "+1y");
	print LOG "$stamp $id New connection\n";
	for my $hdr ( qw(
	    HTTP_ACCEPT HTTP_ACCEPT_CHARSET HTTP_ACCEPT_ENCODING
	    HTTP_ACCEPT_LANGUAGE
	    HTTP_HOST HTTP_USER_AGENT
	    HTTP_VIA HTTP_X_FORWARDED_FOR
	    REMOTE_ADDR REMOTE_PORT
	    ) )
	{
	    if ( $ENV{$hdr} ) {
		print LOG "$stamp $id $hdr = $ENV{$hdr}\n";
	    }
	}
    }
    $id;
}

sub present_questions {
    my($q, $data, $id, $ronde) = @_;

    my $time = time;
    print LOG scalar localtime $time,
	" $id $ronde Retrieves ronde $data->[0]\n";
    print <<HTML;
<p align=center><center><h1>$data->[0]</h1></center></p>
HTML
    print $q->start_form,
	$q->hidden(-name => "ID", -value => $id, -override => 1),
	$q->hidden(-name => "start", -value => $time, -override => 1),
	$q->hidden(-name => "ronde", -value => $ronde, -override => 1),
	"\n";
    for (my $nr = 1; $data->[$nr]; $nr++) {
	# show question
	showq($nr, $data);

	# get answers
	my @buttons = @{$data->[$nr]}[3..6];
	if ( ! $data->[$nr][2] ) {
	    # no fixed answer, shuffle answers
	    permutate(\@buttons);
	}

	# display answers
	print "<p>\n",
	    $q->radio_group(-name => "vraag$nr",
			    '-values' => \@buttons,
			    -default => "-",
			    -linebreak => 1),
	    "</p><hr>\n";
    }

    print $q->submit(-name => "answ", -value => "Controleer antwoorden"),
	$q->end_form,
	"\n";
}

sub showq {
    my($nr, $data) = @_;

    print "<p>Vraag $nr</p>\n";
    print qq{<img border=1 src="$data->[$nr][1]"><br>\n}
	if $data->[$nr][1];
    print "<p>", htmlize($data->[$nr][0]), "</p>\n";
}

sub process_answers {
    my($q, $data, $id, $ronde) = @_;

    my $time = time;
    print LOG scalar localtime $time,
	" $id $ronde Completes ronde $data->[0]\n";
    print LOG scalar localtime $time,
	" $id $ronde did not accept cookie\n" unless $q->cookie(-name => "ID");
    print <<HTML;
<p align=center><center><h1>Antwoorden van $data->[0]</h1></center></p>
HTML

    my $start = $q->param(-name => "start")
	or die "No start";

    my $ok = 0;
    my $wrong = 0;
    my $unansw = 0;

    for (my $nr = 1; $data->[$nr]; $nr++) {
	my $answ = $q->param(-name => "vraag$nr");
	if ( ! $answ ) {
	    print LOG scalar localtime $time,
		" $id $ronde did not answer question $nr\n";
	    print "<p>Vraag $nr niet beantwoord</p><hr>\n";
	    $unansw++;
	    next;
	}

	# repeat question.
	showq($nr, $data);

	# get correct answer
	my $correct = $data->[$nr][ 2 + ($data->[$nr][2] || 1) ];

	my $htmltxt;
	my $logtxt;
	my $gif;
	if ( $answ eq $correct ) {
	    $htmltxt = '<font color="#66FF66">Goed beantwoord</font>';
	    $logtxt = "Correct";
	    $gif = "ok.gif";
	    $ok++;
	}
	else {
	    $htmltxt = qq{U antwoordde: $answ } .
		qq{<font color="#FF6666">Onjuist</font>};
	    $logtxt = "Wrong";
	    $gif = "wrong.gif";
	    $wrong++;
	}

	print qq{<img src="$gif">}, " Juist antwoord: ",
	    htmlize($correct), "\n";

	# find given answer
	my $given;
	for my $a ( 1 .. 4 ) {
	    $given = chr(96+$a)
		if $answ eq $data->[$nr][2 + $a];
	}
	$given ||= "Nothing mathed! ($answ)";
	print LOG scalar localtime $time,
	    " $id $ronde answered on question $nr: $given ($logtxt)\n";
	print "<p>$htmltxt</p>\n";

	# print any explanation
	print htmlize($data->[$nr][7]) if $data->[$nr][7];

	print "<hr>\n";
    }
    my $duration = $time - $start;
    my $total = $ok + $wrong;
    print LOG scalar localtime $time,
	" $id $ronde started on $start\n",
	scalar localtime $time,
	" $id gave $total answers in $duration seconds\n";
    if ( $total ) {
	my $avg = sprintf("%.2f", $duration / $total);
	print LOG scalar localtime $time,
	    " $id $ronde took $avg seconds/question\n";
    }
    print LOG scalar localtime $time,
	" $id $ronde scored $ok/$total/$#$data\n";
    print <<HTML;
<p><b>Score: $ok</b></p>

<a href="index.html">Terug naar de index</a><br>
HTML
    $ronde++;
    if ( -f "ronde$ronde.data" ) {
	print <<HTML;
<a href="quiz.cgi?ronde=$ronde">Volgende ronde</a><br>
HTML
    }
}

sub htmlize {
    my $str = shift;

    # escape dangerous characters
    $str =~ s/\&/&amp;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    $str =~ s/"/&quot;/g;
    # insert linebreaks where necessary
    $str =~ s/\n(\s)/<br>$1/g;
    # insert pagebreaks where necessary
    $str =~ s%\n%</p>\n<p>%g;
    # convert runs of spaces to non-breaking
    $str =~ s/(  +)/"&nbsp;" x length $1/ge;

    $str;
}

sub permutate {
    my $ar = shift;

    for(my $i = $#$ar; $i > 0; $i--) {
	my $j = int rand($i + 1);
	($ar->[$i], $ar->[$j]) = ($ar->[$j], $ar->[$i]) if $i != $j;
    }
}
